package SDON;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * Represents a Cell in a table.
 */
public class Cell implements TextInterfaceCore<Cell>, TextInterfaceExtended<Cell>, TextInterfaceAlignment<Cell> {
    /**
     * Required. The column of the cell. Note that the first column is column 1
     * not column 0.
     */
    @SDONSerializable
    public int Column = -1;
    
    /**
     * Required. The row of the cell. Note that the first row is row 1 not row 0.
     */
    @SDONSerializable
    public int Row = -1;
    
    /**
     * The text label inside the cell.
     */
    @SDONSerializable
    public String Label = null;
    
    /**
     * The text label inside the cell.
     * @param label - Label text to set
     * @return Cell 
     */
    public Cell SetLabel(String label) {
    	Label = label;
    	
    	return this;
    }
    
    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     */
    @SDONSerializable
    public double TextSize = -1.0;
    
    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     * @param textSize - Size of text
     * @return Cell
     */
    public Cell SetTextSize(double textSize) {
    	
    	TextSize = textSize;
    	
    	return this;
    }
    
    /**
     * Makes the text label be bold with a value of true, not bold with false,
     * otherwise, if omitted, boldness follows the template default.
     */
    @SDONSerializable
    public Boolean TextBold = null;
    
    /**
     * Makes the text label be bold with a value of true, not bold with false
     * @param textBold - True for bold text
     * @return Cell
     */
    public Cell SetTextBold(Boolean textBold) {
    	TextBold = textBold;
    	
    	return this;
    }
    
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false, otherwise, if omitted, italic follows the template default.
     */
    @SDONSerializable
    public Boolean TextItalic = null;
    
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false
     * @param textItalic - True for italic text
     * @return Cell
     */
    public Cell SetTextItalic(Boolean textItalic) {
    	
    	TextItalic = textItalic;
    	
    	return this;
    }
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false, otherwise, if omitted, underlined follows the template
     * default.
     */
    @SDONSerializable
    public Boolean TextUnderline = null;
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false
     * @param textUnderline - True for underlined text
     * @return Cell
     */
    public Cell SetTextUnderline(Boolean textUnderline) {
    	
    	TextUnderline = textUnderline;
    	
    	return this;
    }
    
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     */
    @SDONSerializable
    public String TextFont = null;
    
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     * @param textFont - Font name
     * @return Cell
     */
    public Cell SetTextFont(String textFont) {
    	
    	TextFont = textFont;
    	
    	return this;
    }
    
    /**
     * The color of the text label of the cell (hex RGB value or alias). If
     * omitted, the color is the default for the template.
     */
    @SDONSerializable
    public String TextColor = null;
    
    /**
     * Set the color of the text label of the cell (hex RGB value or alias). If
     * omitted, the color is the default for the template.
     * @param textColor - Text color
     * @return Cell
     */
    public Cell SetTextColor(String textColor) {
    	
	   TextColor = textColor;
    	
    	return this;
    }
    
    /**
     * The fill color of the cell (hex RGB value or alias). If omitted, color
     * is the default for the template.
     */
    @SDONSerializable
    public String FillColor = null;
    
    /**
     * The fill color of the shape (hex RGB value or alias). 
     * @param fillColor - Color to fill cell
     * @return Cell
     */
    public Cell SetFillColor(String fillColor) {
    	
    	FillColor = fillColor;
    	
    	return this;
    }    
    
    /**
     * Aligns the text label to the left, right or centered in the cell. If
     * omitted, the alignment is the default for the template. A value from the
     * {@link HorizontalAlignments} enum.
     */
    @SDONSerializable
    public String TextAlignH = null;
    
    /**
     * Aligns the text label to the left, right or centered in the cell.
     * @param textAlignH - One of the SDON.HorizontalAlignments enum
     * @return Cell
     */
    public Cell SetTextAlignH(String textAlignH) {
    	TextAlignH = textAlignH;
    	
    	return this;
    }
    
    /**
     * Align the text label to the top, bottom or middle in the shape. If
     * omitted, the alignment is the default for the template. A value from the
     * {@link VerticalAlignments} enum.
     */
    @SDONSerializable
    public String TextAlignV = null;
    
    /**
     * Align the text label to the top, bottom or middle in the shape.
     * @param textAlignV - One of the SDON.VerticalAlignments enum
     * @return Cell
     */
    public Cell SetTextAlignV(String textAlignV) {
    	TextAlignV = textAlignV;
    	
    	return this;
    }
    
    /**
     * The number of characters shown in the cell before the text is truncated.
     */
    @SDONSerializable
    public int Truncate = -1;
    
    /**
     * The number of characters shown in the cell before the text is truncated.
     * @param truncate - Number of characters before truncation
     * @return Cell
     */
    public Cell SetTextTruncate(int truncate) {
    	Truncate = truncate;
    	
    	return this;
    }
    
    /**
     * A string to appear as a note attached to a cell.
     */
    @SDONSerializable
    public String Note = null;
    
    /**
     *  A string to appear as a note attached to a cell.
     * @param note - Text of note
     * @return Cell
     */
    public Cell SetNote(String note) {
    	Note = note;
   	
    	return this;
   }
    
    /**
     * The icon to use for the note on a cell. Defaults to a post-it note icon.
     * Other options are available in the {@link Icons} enum.
     */
    @SDONSerializable
    public String NoteIcon = null;
    
    /**
     * The icon to use for the note on a cell. Defaults to a post-it note icon.
     * Other options are available in the {@link Icons} enum.
     * @param noteIcon - Note icon name
     * @return Cell
     */
    public Cell SetNoteIcon(String noteIcon) {
    	NoteIcon = noteIcon;
   	
    	return this;
   }    
    
    /**
     * A hyperlink in the cell.
     */
    @SDONSerializable
    public Hyperlink Hyperlink = null;
    
    /**
     * Set the hyperlink in a cell
     * @param url - Hyperlink url
     * @return Cell
     */
    public Cell SetHyperlink(String url) {
    	Hyperlink = new Hyperlink();
    	
    	Hyperlink.url = url;
    	
    	return this;
    }
    
    /**
     * A hyperlink for the text in the cell.
     */
    @SDONSerializable
    public Hyperlink TextHyperlink = null;
    
    /**
     * Set a hyperlink for cell text
     * @param url - Hyperlink url
     * @return Cell
     */
    public Cell SetTextHyperlink(String url) {
    	TextHyperlink = new Hyperlink();
    	
    	TextHyperlink.url = url;
    	
    	return this;
    }
    
    /**
     * Defines the url to the image to be shown in the cell.
     */
    @SDONSerializable
    public Image Image = null;
    
    /**
     * Set an image to be displayed in the cell
     * @param url - Url to image to display
     * @return Cell
     */
    public Cell SetImage(String url) {
    	Image = new Image();
    	
    	Image.url = url;
    	
    	return this;
    }
    
    /**
     * A shape that appears inside a table cell. The cell will resize to
     * accomodate the shape and its children.
     */
    @SDONSerializable
    public Shape Shape = null;
    
    /**
     * A shape that appears inside a table cell. The cell will resize to
     * accomodate the shape and its children.
     * @return Shape
     */
    public Shape AddShape() {
    	if (Shape != null) 
    		return Shape;
    	
    	Shape = new Shape();
    	
    	return Shape;
    }
}