package SDON;

import java.util.ArrayList;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * 
 */
public class DataTable {
    /**
     * The unique identifier for the data table so it can be referenced in a
     * shape object.
     */
    @SDONSerializable
    public String ID = null;
    
    /**
     * The name to give the table.
     */
    @SDONSerializable
    public String TableName = null;
    
    /**
     * The list of column definitions in a data table. The schema of the data
     * table.
     */
    @SDONSerializable
    public ArrayList<DataColumn> Columns = null;
    
    /**
     * The list of column definitions in a data table. The schema of the data
     * table.
     * @param columns - DataColumn list
     * @return DataTable
     */
    public DataTable SetColumns(ArrayList<DataColumn> columns) {
    	if (Columns == null)
    		Columns = new ArrayList<DataColumn>();
    	
    	for (int i = 0; i < columns.size(); i++) {
    		Columns.add(columns.get(i));
    	}
    	
    	return this;
    }
    
    /**
     * Optional. The data to populate rows in the data table with.
     */
    @SDONSerializable
    public ArrayList<DataRow> Rows = null;
    
    /**
     * Add a row of data to the table
     * @return DataRow
     */
    public DataRow AddDataRow() {
    	if (Rows == null)
    		Rows = new ArrayList<DataRow>();
    	
    	String id = String.valueOf(Rows.size() + 1);
    	
    	return AddDataRow(id);
    }
    
    /**
     * Add a row of data to the table
     * @param id - ID to set in new row
     * @return DataRow
     */
    public DataRow AddDataRow(String id) {
    	if (Rows == null)
    		Rows = new ArrayList<DataRow>();
    	
    	for (DataRow row : Rows) {
    		if (row.RowID == id)
    			return row;
    	}
    	
    	DataRow dataRow = new DataRow();
    	dataRow.RowID = id;
    	
    	Rows.add(dataRow);
    	
    	return dataRow;
    }
    
    /**
     * Retrieve a data row object by ID
     * @param rowID - ID of row to retrieve
     * @return DataRow
     */
    public DataRow GetDataRow(String rowID) {
    	if (Rows == null)
    		return null;
    	
    	for (DataRow r : Rows) {
    		if (r.RowID == rowID)
    			return r;
    	}
    	
    	return null;
    }
}