package SDON;

import java.util.ArrayList;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * Object that contains a reference to a DataTable defined in the Diagram
 * object. Used to fill out the schema defined in the DataTable array in the
 * Diagram object.
 */
public class DataTableInstance {
    /**
     * The name of a predefined table in the template used.
     */
    @SDONSerializable
    public String TableName = null;
    
    /**
     * The rows to insert in the pre-existing table.
     */
    @SDONSerializable
    public ArrayList<DataRow> Rows = null;
    
    /**
     * Add a data row
     * @return DataRow
     */
    public DataRow AddDataRow() {
    	if (Rows == null)
    		Rows = new ArrayList<DataRow>();
    	
    	String id = String.valueOf(Rows.size() + 1);
    	
    	return AddDataRow(id);
    }
    
    /**
     * Add a data row
     * @param id - ID of new data row
     * @return DataRow
     */
    public DataRow AddDataRow(String id) {
    	if (Rows == null)
    		Rows = new ArrayList<DataRow>();
    	
    	for (DataRow row : Rows) {
    		if (row.RowID == id)
    			return row;
    	}
    	
    	DataRow dataRow = new DataRow();
    	dataRow.RowID = id;
    	
    	Rows.add(dataRow);
    	
    	return dataRow;
    }
}