package SDON;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * An object representing a segmented line that links two shapes.
 */
public class Return implements LineInterface<Return>, TextInterfaceCore<Return>, TextInterfaceExtended<Return> {
    /**
     * Required. The ID of the starting shape. The starting and ending shapes
     * define the direction of the line as far as any arrowhead is concerned,
     * the arrowhead touches the ending shape.
     */
    @SDONSerializable
    public int StartID = -1;
    
    /**
     * Required. The ID of the ending shape. The starting and ending shapes
     * define the direction of the line as far as any arrowhead is concerned,
     * the arrowhead touches the ending shape.
     */
    @SDONSerializable
    public int EndID = -1;
    
    /**
     * The direction the line leaves the starting shape. The default is Down. A
     * value from the {@link Directions} enum.
     */
    @SDONSerializable
    public String StartDirection = null;
    
    /**
     * The direction the line leaves the starting shape. The default is Down. A
     * value from the {@link Directions} enum.
     * @param startDirection - Start direction
     * @return Return
     */
    public Return SetStartDirection(String startDirection) {
    	StartDirection = startDirection;
   	
   	return this;
   }
    
    /**
     * The direction the line goes out the ending shape. The default is Down. A
     * value from the {@link Directions} enum.
     */
    @SDONSerializable
    public String EndDirection = null;
    
    /**
     * The direction the line goes out the ending shape. The default is Down. A
     * value from the {@link Directions} enum.
     * @param endDirection - End direction
     * @return Return
     */
    public Return SetEndDirection(String endDirection) {
    	EndDirection = endDirection;
   	
   	return this;
   }
    
    /**
     * The pattern of the line connecting the shapes. A value from the
     * {@link LinePatterns} enum.
     */
    @SDONSerializable
    public String LinePattern = null;
    
    /**
     * The pattern of the line connecting the shapes. A value from the
     * {@link LinePatterns} enum.
     * @param linePattern - Line pattern
     * @return Return
     */
    public Return SetLinePattern(String linePattern) {
    	LinePattern = linePattern;
   	
   	return this;
   }
    
    /**
     * The text that appears on the line.
     */
    @SDONSerializable
    public String Label = null;
    
    /**
     * The text that appears on the line.
     * @param label - Label string
     * @return Return
     */
    public Return SetLabel(String label) {
    	Label = label;
   	
   	return this;
   }
    
    /**
     * Makes the text label be bold with a value of true, not bold with false,
     * otherwise, if omitted, boldness follows the template default.
     */
    @SDONSerializable
    public Boolean TextBold = null;
    
    /**
     * Makes the text label be bold with a value of true, not bold with false,
     * otherwise, if omitted, boldness follows the template default.
     * @param textBold - Bold text
     * @return Return
     */
    public Return SetTextBold(Boolean textBold) {
    	TextBold = textBold;
    	
    	return this;
    }
            
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false, otherwise, if omitted, italic follows the template default.
     */
    @SDONSerializable
    public Boolean TextItalic = null;
    
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false, otherwise, if omitted, italic follows the template default.
     * @param textItalic - Text in italic
     * @return Shape
     */
    public Return SetTextItalic(Boolean textItalic) {
    	TextItalic = textItalic;
    	
    	return this;
    }
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false, otherwise, if omitted, underlined follows the template
     * default.
     */
    @SDONSerializable
    public Boolean TextUnderline = null;
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false, otherwise, if omitted, underlined follows the template
     * default.
     * @param textUnderline - Text underlined
     * @return Return
     */
    public Return SetTextUnderline(Boolean textUnderline) {
    	TextUnderline = textUnderline;
    	
    	return this;
    }
    
    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     */
    @SDONSerializable
    public double TextSize = -1.0;

    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     * @param textSize - Text size
     * @return Return
     */
    public Return SetTextSize(double textSize) {
    	TextSize = textSize;
    	
    	return this;
    }
    
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     */
    @SDONSerializable
    public String TextFont = null;
    
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     * @param textFont - Font name
     * @return Return
     */
    public Return SetTextFont(String textFont) {
    	TextFont = textFont;
    	
    	return this;
    }
   
    
    /**
     * The color of the text label of the return (hex RGB value or color alias).
     * If omitted, the color is the default for the template.
     */
    @SDONSerializable
    public String TextColor = null;
    
    /**
     * The color of the text label of the return (hex RGB value or color alias).
     * If omitted, the color is the default for the template.
     * @param textColor - Text color
     * @return Return
     */
    public Return SetTextColor(String textColor) {
    	TextColor = textColor;
    	
    	return this;
    }
    
    /**
     * A hyperlink on the text in the return.
     */
    @SDONSerializable
    public Hyperlink TextHyperlink = null;
    
    /**
     * A hyperlink on the text in the return.
     * @param textHyperlink - URL
     * @return Return
     */
    public Return SetTextHyperlink(String  textHyperlink) {
    	Hyperlink hl = new Hyperlink();
    	
    	hl.url = textHyperlink;
    	
    	TextHyperlink = hl;
    	
    	return this;
    }
    
    /**
     * Defines the number of characters to allow before the
     * remaining text is truncated. By default nothing is truncated. Defining
     * Truncate to "-1" turns it off if on by default.
     */
    @SDONSerializable
    public int Truncate = -1;
    
    /**
     * Defines the number of characters to allow in a return before the
     * remaining text is truncated. By default nothing is truncated. Defining
     * Truncate to "-1" turns it off if on by default.
     * @param truncate - Number of characters to display before truncating text
     * @return Return
     */
    public Return SetTextTruncate(int  truncate) {
    	Truncate = truncate;
    	
    	return this;
    }
    
    
    /**
     * The arrowhead that will appear on the beginning of the line. A value
     * from the {@link Arrowheads} enum. The default arrowhead depends on
     * the template used.
     */
    @SDONSerializable
    public int StartArrow = -1;
    
    /**
     * The arrowhead that will appear on the beginning of the line. A value
     * from the {@link Arrowheads} enum. The default arrowhead depends on
     * the template used.
     * @param startArrow - Start arrow value
     * @return Return
     */
    public Return SetStartArrow(int startArrow) {
    	StartArrow = startArrow;
   	
   	return this;
   }
    
    /**
     * The arrowhead that will appear on the end of the line. A value from the
     * {@link Arrowheads} enum. The default arrowhead depends on the template
     * used.
     */
    @SDONSerializable
    public int EndArrow = -1;
    
    /**
     * The arrowhead that will appear on the end of the line. A value from the
     * {@link Arrowheads} enum. The default arrowhead depends on the template
     * used.
     * @param endArrow - End arrow value
     * @return Return
     */
    public Return SetEndArrow(int endArrow) {
    	EndArrow = endArrow;
   	
   	return this;
   }
    
    /**
     * The thickness of the line in 1/100". Otherwise the thickness is the
     * default for the template.
     */
    @SDONSerializable
    public double LineThick = -1.0;
    
    /**
     * The thickness of the line in 1/100". Otherwise the thickness is the
     * default for the template.
     * @param lineThick - Line thickness
     * @return Return
     */
    public Return SetLineThickness(double lineThick) {
    	LineThick = lineThick;
   	
   	return this;
   }
    
    /**
     * Make the line color of the connector the specified RGB value or color
     * alias. Otherwise the color is the default for the template.
     */
    @SDONSerializable
    public String LineColor = null;
    
    /**
     * Make the line color of the connector the specified RGB value or color
     * alias. Otherwise the color is the default for the template.
     * @param lineColor - Line color
     * @return Return
     */
    public Return SetLineColor(String lineColor) {
    	LineColor = lineColor;
   	
   	return this;
   }
    
    /**
     * When set to true this creates a curved return.
     */
    @SDONSerializable
    public Boolean Curved = null;
    
    /**
     * When set to true this creates a curved return.
     * @param curved - Curved line
     * @return Return
     */
    public Return Curved(Boolean curved) {
    	Curved = curved;
   	
   	return this;
   }
}