package SDON;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * Object for containing the properties of a row in a table.
 */
public class RowProperties implements LineInterface<RowProperties> {
    /**
     * The index of the row.
     */
    @SDONSerializable
    public int Index = -1;
    
    /**
     * The thickness of the row borders in 1/100". If omitted, the thickness is
     * the default for the template.
     */
    @SDONSerializable
    public double LineThick = -1.0;
    
    /**
     * The thickness of the row borders in 1/100". If omitted, the thickness is
     * the default for the template.
     * @param lineThick - Line thickness
     * @return RowProperties
     */
    public RowProperties SetLineThickness(double lineThick) {
    	LineThick = lineThick;
   	
   	return this;
   }
    
    /**
     * The line color of the row borders as a hex RGB value or color alias. If
     * omitted, the color is the default for the template.
     */
    @SDONSerializable
    public String LineColor = null;
    
    /**
     * The line color of the row borders as a hex RGB value or color alias. If
     * omitted, the color is the default for the template.
     * @param lineColor
     * @return RowProperties
     */
    public RowProperties SetLineColor(String lineColor) {
    	LineColor = lineColor;
   	
   	return this;
   }
    
    /**
     * The pattern of the line for the row border. Must be a value from the
     * {@link LinePatterns} enum.
     */
    @SDONSerializable
    public String LinePattern = null;
    
    /**
     * The pattern of the line for the row border. Must be a value from the
     * {@link LinePatterns} enum.
     * @param linePattern - Line pattern
     * @return RowProperties
     */
    public RowProperties SetLinePattern(String linePattern) {
    	LinePattern = linePattern;
   	
   	return this;
   }
    
    /**
     * The minimum height of the row in 1/100". The text in cells in the row
     * may force the row to be taller than this height.
     */
    @SDONSerializable
    public double Height = -1.0;
    
    /**
     * The minimum height of the row in 1/100". The text in cells in the row
     * may force the row to be taller than this height.
     * @param height - Row Height
     * @return RowProperties
     */
    public RowProperties SetHeight(double height) {
    	Height = height;
   	
   	return this;
   }
    
    /**
     * Makes the row not scale as the shape that contains the table grows.
     */
    @SDONSerializable
    public Boolean FixedHeight = null;
    
    /**
     * Makes the row not scale as the shape that contains the table grows.
     * @param fixedHeight - Make row a fixed height
     * @return RowProperties
     */
    public RowProperties FixedHeight(Boolean fixedHeight) {
    	FixedHeight = fixedHeight;
   	
   	return this;
   }
}