package SDON;

import java.util.ArrayList;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * Object that represents a shape in a SmartDraw diagram.
 */
public class Shape implements LineInterface<Shape>, TextInterfaceCore<Shape>, TextInterfaceExtended<Shape>, TextInterfaceAlignment<Shape> {
    /**
     * The ID of this shape. IDs are used to specify the paths of Return lines.
     * IDs arbitrary but should be unique and must be greater than zero.
     */
    @SDONSerializable
    public int ID = -1;
    
    /**
     * The ID of this shape. IDs are used to specify the paths of Return lines.
     * IDs arbitrary but should be unique and must be greater than zero.
     * @param id - ID to set on shape
     * @return Shape
     */
    public Shape SetID(int id) {
    	ID = id;
    	
    	return this;
    }
    
    /**
     * The text label inside the shape.
     */
    @SDONSerializable
    public String Label = null;
    
    /**
     * The text label inside the shape.
     * @param label - Label text
     * @return Shape
     */
    public Shape SetLabel(String label) {
    	Label = label;
    	
    	return this;
    }
    
    /**
     * The type of shape. Used to change the shape from a rectangle (default)
     * to another shape. A value from the {@link ShapeTypes} enum or a value
     * from the containing diagram's symbol table.
     */
    @SDONSerializable
    public String ShapeType = null;
    
    /**
     * The type of shape. Used to change the shape from a rectangle (default)
     * to another shape. A value from the {@link ShapeTypes} enum or a value
     * from the containing diagram's symbol table.
     * @param shapeType - Shape type
     * @return Shape
     */
    public Shape SetShapeType(String shapeType) {
    	ShapeType = shapeType;
    	
    	return this;
    }
    
    /**
     * Makes the text label be bold with a value of true, not bold with false,
     * otherwise, if omitted, boldness follows the template default.
     */
    @SDONSerializable
    public Boolean TextBold = null;
    
    /**
     * Makes the text label be bold with a value of true, not bold with false,
     * otherwise, if omitted, boldness follows the template default.
     * @param textBold - Bold text
     * @return Shape
     */
    public Shape SetTextBold(Boolean textBold) {
    	TextBold = textBold;
    	
    	return this;
    }
    
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false, otherwise, if omitted, italic follows the template default.
     */
    @SDONSerializable
    public Boolean TextItalic = null;
    
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false, otherwise, if omitted, italic follows the template default.
     * @param textItalic - Text in italic
     * @return Shape
     */
    public Shape SetTextItalic(Boolean textItalic) {
    	TextItalic = textItalic;
    	
    	return this;
    }
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false, otherwise, if omitted, underlined follows the template
     * default.
     */
    @SDONSerializable
    public Boolean TextUnderline = null;
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false, otherwise, if omitted, underlined follows the template
     * default.
     * @param textUnderline - Text underlined
     * @return Shape
     */
    public Shape SetTextUnderline(Boolean textUnderline) {
    	TextUnderline = textUnderline;
    	
    	return this;
    }
    
    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     */
    @SDONSerializable
    public double TextSize = -1.0;

    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     * @param textSize - Text size
     * @return Shape
     */
    public Shape SetTextSize(double textSize) {
    	TextSize = textSize;
    	
    	return this;
    }
    
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     */
    @SDONSerializable
    public String TextFont = null;
    
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     * @param textFont - Font name
     * @return Shape
     */
    public Shape SetTextFont(String textFont) {
    	TextFont = textFont;
    	
    	return this;
    }
    
    /**
     * Sets the space (in 100 dpi) between the text and the outside edge of the
     * shape.
     */
    @SDONSerializable
    public int TextMargin = -1;
    
    /**
     * Sets the space (in 100 dpi) between the text and the outside edge of the
     * shape.
     * @param textMargin - Text margin
     * @return Shape
     */
    public Shape SetTextMargin(int textMargin) {
    	TextMargin = textMargin;
    	
    	return this;
    }
    
    /**
     * The color of the text label of the shape (hex RGB value or color alias).
     * If omitted, the color is the default for the template.
     */
    @SDONSerializable
    public String TextColor = null;
    
    /**
     * The color of the text label of the shape (hex RGB value or color alias).
     * If omitted, the color is the default for the template.
     * @param textColor - Text color
     * @return Shape
     */
    public Shape SetTextColor(String textColor) {
    	TextColor = textColor;
    	
    	return this;
    }
    
    /**
     * The fill color of the shape (hex RGB value or color alias). If omitted,
     * color is the default for the template.
     */
    @SDONSerializable
    public String FillColor = null;
    
    /**
     * The fill color of the shape (hex RGB value or color alias). If omitted,
     * color is the default for the template.
     * @param fillColor - Fill color
     * @return Shape
     */
    public Shape SetFillColor(String  fillColor) {
    	FillColor = fillColor;
    	
    	return this;
    }
    
    /**
     * Aligns the text label to the left, right or centered in the shape. If
     * omitted, the alignment is the default for the template. A value from the
     * {@link HorizontalAlignments} enum.
     */
    @SDONSerializable
    public String TextAlignH = null;
    
    /**
     * Aligns the text label to the left, right or centered in the shape. If
     * omitted, the alignment is the default for the template. A value from the
     * {@link HorizontalAlignments} enum.
     * @param textAlignH - Horizontal text alignment
     * @return Shape
     */
    public Shape SetTextAlignH(String  textAlignH) {
    	TextAlignH = textAlignH;
    	
    	return this;
    }
    
    /**
     * Align the text label to the top, bottom or middle in the shape. If
     * omitted, the alignment is the default for the template. A value from the
     * {@link VerticalAlignments} enum.
     */
    @SDONSerializable
    public String TextAlignV = null;
    
    /**
     * Align the text label to the top, bottom or middle in the shape. If
     * omitted, the alignment is the default for the template. A value from the
     * {@link VerticalAlignments} enum.
     * @param textAlignV - Vertical text alignment
     * @return Shape
     */
    public Shape SetTextAlignV(String  textAlignV) {
    	TextAlignV = textAlignV;
    	
    	return this;
    }
    
    /**
     * Determines the way in which a shape will grow as text is added. Values
     * must be from the {@link TextGrow} enum.
     */
    @SDONSerializable
    public String TextGrow = null;
    
    /**
     * Determines the way in which a shape will grow as text is added. Values
     * must be from the {@link TextGrow} enum.
     * @param textGrow - Text Grow direction
     * @return Shape
     */
    public Shape SetTextGrow(String textGrow) {
    	TextGrow = textGrow;
    	
    	return this;
    }
    
    /**
     * Specifies the initial width of a shape in 1/100" before any text is
     * added. Adding more text than will fit into the shape will grow it
     * according to the TextGrow value.
     */
    @SDONSerializable
    public double MinWidth = -1.0;
    
    /**
     * Specifies the initial width of a shape in 1/100" before any text is
     * added. Adding more text than will fit into the shape will grow it
     * according to the TextGrow value.
     * @param minWidth - Minimum width
     * @return Shape
     */
    public Shape SetMinWidth(double minWidth) {
    	MinWidth = minWidth;
    	
    	return this;
    }
    
    /**
     * Specifies the initial height of a shape in 1/100" before any text is
     * added. Adding more text than will fit into the shape will grow it
     * according to the TextGrow value.
     */
    @SDONSerializable
    public double MinHeight = -1.0;
    
    /**
     * Specifies the initial height of a shape in 1/100" before any text is
     * added. Adding more text than will fit into the shape will grow it
     * according to the TextGrow value.
     * @param minHeight - Minimum height
     * @return Shape
     */
    public Shape SetMinHeight(double  minHeight) {
    	MinHeight = minHeight;
    	
    	return this;
    }
    
    /**
     * The border thickness of the shape in 1/100". If omitted, the thickness
     * is the default for the template.
     */
    @SDONSerializable
    public double LineThick = -1.0;
    
    /**
     * The border thickness of the shape in 1/100". If omitted, the thickness
     * is the default for the template.
     * @param lineThick - Line thickneess
     * @return Shape
     */
    public Shape SetLineThickness(double  lineThick) {
    	LineThick = lineThick;
    	
    	return this;
    }
    
    /**
     * The border color of the shape (hex RGB value or color alias). If
     * omitted, color is the default for the template.
     */
    @SDONSerializable
    public String LineColor = null;
    
    /**
     * The border color of the shape (hex RGB value or color alias). If
     * omitted, color is the default for the template.
     * @param lineColor - Line color
     * @return Shape
     */
    public Shape SetLineColor(String  lineColor) {
    	LineColor = lineColor;
    	
    	return this;
    }
    
    /**
     * A text label on the connector line segment that touches the shape.
     */
    @SDONSerializable
    public String LineLabel = null;
    
    /**
     * A text label on the connector line segment that touches the shape.
     * @param lineLabel - Line label
     * @return Shape
     */
    public Shape SetLineLabel(String  lineLabel) {
    	LineLabel = lineLabel;
    	
    	return this;
    }
    
    /**
     * The pattern of the line for the connector. Must be a value from the
     * {@link LinePatterns} enum.
     */
    @SDONSerializable
    public String LinePattern = null;
    
    /**
     * The pattern of the line for the connector. Must be a value from the
     * {@link LinePatterns} enum.
     * @param linePattern - Line pattern
     * @return Shape
     */
    public Shape SetLinePattern(String  linePattern) {
    	LinePattern = linePattern;
    	
    	return this;
    }
    
    /**
     * Whether or not to display the shape if this shape is a parent
     * ShapeContainer to child shapes.
     */
    @SDONSerializable
    public Boolean Hide = null;
    
    /**
     * Whether or not to display the shape if this shape is a parent
     * ShapeContainer to child shapes.
     * @param hide - Hide shape
     * @return Shape
     */
    public Shape Hide(Boolean hide  ) {
    	Hide = hide;
    	
    	return this;
    }
    
    
    /**
     * Defines the type of connector that is coming off of this shape is the
     * parent of. Must be a value from {@link ShapeConnectorTypes}.
     */
    @SDONSerializable
    public String ShapeConnectorType = null;
    
    /**
     * A string to appear as a note attached to a shape.
     */
    @SDONSerializable
    public String Note = null;
    
    /**
     * A string to appear as a note attached to a shape.
     * @param note - note text
     * @return Shape
     */
    public Shape SetNote(String note) {
    	Note = note;
    	
    	return this;
    }
    
    /**
     * The icon to use for the note on a shape. Defaults to a post-it note
     * icon. Other options are available in the {@link Icons} enum.
     */
    @SDONSerializable
    public String NoteIcon = null;
    
    /**
     * The icon to use for the note on a shape. Defaults to a post-it note
     * icon. Other options are available in the {@link Icons} enum.
     * @param noteIcon - Note icon
     * @return Shape
     */
    public Shape SetNoteIcon(String noteIcon) {
    	NoteIcon = noteIcon;
    	
    	return this;
    }
    
    /**
     * The list of ShapeConnectors that are attached to and branch off from
     * this shape.
     */
    @SDONSerializable
    public ArrayList<ShapeConnector> ShapeConnector = null;
    
    /**
     * Add a ShapeConnector to The list of ShapeConnectors that are attached to and branch off from
     * this shape.
     * @return ShapeConnector
     */
    public ShapeConnector AddShapeConnector( ) {
    	if (ShapeConnector == null) {
    		ShapeConnector = new ArrayList<ShapeConnector>();
    	}
    	
    	ShapeConnector shapeConnector = new ShapeConnector();
    	
    	ShapeConnector.add(shapeConnector);
    	
    	return shapeConnector;
    }
    
    /**
     * A hyperlink on the shape.
     */
    @SDONSerializable
    public Hyperlink Hyperlink = null;
    
    /**
     * A hyperlink on the shape.
     * @param hyperlink - URL
     * @return Shape
     */
    public Shape SetHyperlink(String  hyperlink) {
    	Hyperlink hl = new Hyperlink();
    	
    	hl.url = hyperlink;
    	
    	Hyperlink = hl;
    	
    	return this;
    }
    
    /**
     * A hyperlink on the text in the shape.
     */
    @SDONSerializable
    public Hyperlink TextHyperlink = null;
    
    /**
     * A hyperlink on the text in the shape.
     * @param textHyperlink - URL
     * @return Shape
     */
    public Shape SetTextHyperlink(String  textHyperlink) {
    	Hyperlink hl = new Hyperlink();
    	
    	hl.url = textHyperlink;
    	
    	TextHyperlink = hl;
    	
    	return this;
    }
    
    /**
     * Defines the number of characters to allow in a shape before the
     * remaining text is truncated. By default nothing is truncated. Defining
     * Truncate to "-1" turns it off if on by default.
     */
    @SDONSerializable
    public int Truncate = -1;
    
    /**
     * Defines the number of characters to allow in a shape before the
     * remaining text is truncated. By default nothing is truncated. Defining
     * Truncate to "-1" turns it off if on by default.
     * @param truncate - Number of characters to display before truncating text
     * @return Shape
     */
    public Shape SetTextTruncate(int  truncate) {
    	Truncate = truncate;
    	
    	return this;
    }
    
    /**
     * Divides the shape up into rows and columns.
     */
    @SDONSerializable
    public Table Table = null;
    
    /**
     * Add a table to this shape
     * @param rows - Number of rows in the table
     * @param columns - Number of oclumns in the table
     * @return Table
     */
    public Table AddTable(int rows, int columns ) {
    	Table = new Table();
    	
    	Table.Rows = rows;
    	Table.Columns = columns;
    	
    	return Table;
    }
    
    /**
     * An image put into the shape.
     */
    @SDONSerializable
    public Image Image = null;
    
    /**
     * Add an image to the shape.
     * @param image - Image URL
     * @return Shape
     */
    public Shape SetImage(String image) {
    	Image = new Image();
    	
    	Image.url = image;
    	
    	return this;
    }
    
    /**
     * Defines arrangements of shapes not connected by lines, but contained
     * inside a parent shape in rows, columns or a matrix arrangement.
     */
    @SDONSerializable
    public ShapeContainer ShapeContainer = null;
    
    /**
     * Add a ShapeContainer to this shape.  Defines an arrangements of shapes not connected by lines, but contained
     * inside a parent shape in rows, columns or a matrix arrangement.
     * @param shapeContainerArrangement - Shape Container arrangement
     * @return ShapeContainer
     */
    public ShapeContainer AddShapeContainer(String  shapeContainerArrangement) {
    	ShapeContainer shapeContainer = new ShapeContainer();
    	
    	shapeContainer.SetArrangement(shapeContainerArrangement);
    	
    	return shapeContainer;
    }
    
    /**
     * An instance of a data table that is used to attach data to a shape.
     */
    @SDONSerializable
    public DataTableShapeEntry Data = null;
    
    /**
     * An instance of a data table that is used to attach data to/associate date with a shape.
     * @param tableID - Table ID
     * @param rowID - Row ID of data associated with this shape
     * @return Shape
     */
    public Shape SetShapeDataRow(String tableID, String rowID) {
    	if (Data == null)
    		Data = new DataTableShapeEntry();
    	
    	Data.TableID = tableID;
    	
    	Data.RowID = rowID;
    	
    	return this;
    }
    

    /**
     * Add a row to a data table and set its information in the shape to associate the data row with the shape
     * @param document - Document object
     * @param tableID - TableID
     * @return DataRow
     */
    public DataRow AddShapeDataRow(Document document, String tableID) {
    	if (document.DataTable == null)
    		return null;
    				
    	DataTable dataTable = null;
    	
    	for (int i = 0; i < document.DataTable.size(); i++) {
    		if (document.DataTable.get(i).ID == tableID) {
    			dataTable = document.DataTable.get(i);
    			break;
    		}
    	}

    	if (dataTable == null)
    		return null;
    	
    	DataRow dataRow = dataTable.AddDataRow();
    	
    	Data = new DataTableShapeEntry();
    	Data.TableID = tableID;
    	Data.RowID = dataRow.RowID;
    
    	return dataRow;    			
    }
}