package SDON;

import java.util.ArrayList;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * Represents a container of shapes arranged in a grid.
 */
public class ShapeContainer {
    /**
     * The array of shapes to include in the group.
     */
    @SDONSerializable
    public ArrayList<Shape> Shapes = null;
    
    /**
     * Add a shape to the array of shapes to include in the group.
     * @return Shape
     */
    public Shape AddShape() {
    	if (Shapes == null)
    		Shapes = new ArrayList<Shape>();
    	
    	Shape shape = new Shape();
    	
    	Shapes.add(shape);
    	
    	return shape;
    }
    
    
    /**
     * The arrangement grid pattern. Must be a value from
     * {@link ShapeContainerArrangement}.
     */
    @SDONSerializable
    public String Arrangement = null;
    
    /**
     * Set the arrangement grid pattern. Must be a value from
     * {@link ShapeContainerArrangement}.
     * @param arrangement - Shape container arrangement
     * @return ShapeContainer
     */
    public ShapeContainer SetArrangement(String arrangement) {
    	Arrangement = arrangement;
    	
    	return this;
    }
    
    /**
     * This is the maximum number of rows for a "Row" arrangement before it
     * wraps to a new row, or the maximum number of columns for a Column
     * arrangement before it wraps to a new column.
     */
    @SDONSerializable
    public int Wrap = -1;
    
    /**
     * Set the maximum number of rows for a "Row" arrangement before it
     * wraps to a new row, or the maximum number of columns for a Column
     * arrangement before it wraps to a new column.
     * @param wrap - Max number of rows
     * @return ShapeContainer
     */
    public ShapeContainer SetWrap(int wrap) {
    	Wrap = wrap;
    	
    	return this;
    }
    
    /**
     * Spacing between rows in 1/100". This is inherited by child
     * ShapeContainers.
     */
    @SDONSerializable
    public int VerticalSpacing = -1;
    
    /**
     * Set the spacing between rows in 1/100". This is inherited by child
     * ShapeContainers.
     * @param verticalSpacing - Vertical Spacing
     * @return ShapeContainer
     */
    public ShapeContainer SetVerticalSpacing(int verticalSpacing) {
    	VerticalSpacing = verticalSpacing;
    	return this;
    }
    
    /**
     * Spacing between columns in 1/100". This is inherited by child
     * ShapeContainers.
     */
    @SDONSerializable
    public int HorizontalSpacing = -1;
    
    /**
     * Set the spacing between columns in 1/100". This is inherited by child
     * ShapeContainers.
     * @param horizontalSpacing - Horizontal spacing
     * @return ShapeContainer
     */
    public ShapeContainer SetHorizontalSpacing(int horizontalSpacing) {
    	HorizontalSpacing = horizontalSpacing;
    	return this;
    }    
    
    /**
     * Controls the positioning of shapes in a column of shapes. Must be a
     * value from {@link HorizontalAlignments}.
     */
    @SDONSerializable
    public String ShapesAlignH = null;
    
    /**
     * Set the positioning of shapes in a column of shapes. Must be a
     * value from {@link HorizontalAlignments}.
     * @param shapesAlignH - Horizontal alignment
     * @return ShapeContainer
     */
    public ShapeContainer SetAlignH(String shapesAlignH) {
    	
    	ShapesAlignH = shapesAlignH;
    	return this;
    }
    
    /**
     * Controls the positioning of a shape in a row of shapes. Must be a value
     * from {@link VerticalAlignments}.
     */
    @SDONSerializable
    public String ShapesAlignV = null;
    
    /**
     * Set the positioning of a shape in a row of shapes. Must be a value
     * from {@link VerticalAlignments}.
     * @param shapesAlignV - Vertical alignment
     * @return ShapeContainer
     */
    public ShapeContainer SetAlignV(String shapesAlignV) {
    	
    	ShapesAlignV = shapesAlignV;
    	return this;
    }
    
    /**
     * The default properties for any shape in this container.
     */
    @SDONSerializable
    public Shape DefaultShape = null;
    
    /**
     * Set the default properties for any shape in this container.
     * @return Shape
     */
    public Shape SetDefaultShape() {
    	if (DefaultShape != null)
    		return DefaultShape;
    	
    	DefaultShape = new Shape();
    	
    	return DefaultShape;
    }
}