package SDON;

import java.util.ArrayList;


/**
 * An object that contains an array of shapes that are connected to it by an
 * automatic connector. Defines an automatic connector.
 */
public interface TextInterfaceCore<T extends TextInterfaceCore<T>> {
 
	
	/**
	 * Set text value
	 * @param label - text
	 * @return T
	 */
	 public T SetLabel(String label);
    
    /**
     * Makes the text label be bold with a value of true, not bold with false,
     * otherwise, if omitted, boldness follows the template default.
     * @param textBold - Bold text
     * @return T
     */
    public T SetTextBold(Boolean textBold);
    
    /**
     * Makes the text label be italic with a value of true, not italic with
     * false, otherwise, if omitted, italic follows the template default.
     * @param textItalic - Italic text
     * @return T
     */
    public T SetTextItalic(Boolean textItalic);
    
    /**
     * Makes the text label be underlined with a value of true, not underlined
     * with false, otherwise, if omitted, underlined follows the template
     * default.
     * @param textUnderline - Underline the text
     * @return T
     */
    public T SetTextUnderline(Boolean textUnderline);
    

    /**
     * The point size of the text label the specified value. If omitted, the
     * text size is the default for the template.
     * @param textSize - Text size
     * @return T
     */
    public T SetTextSize(double textSize);
        
    /**
     * The font of the text label the specified value. If omitted, the font is
     * the default for the template. Any font can be defined, but will fall
     * back to system default if font is unavailable.
     * @param textFont - Font name
     * @return T
     */
    public T SetTextFont(String textFont);
    

    /**
     * The color (hex RGB or color alias) of the text label.
     * @param textColor - Text color
     * @return ShapeConnector
     */
    public T SetTextColor(String textColor);

}